unit DataSetSource;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Db, DBClient, ExtCtrls;

type
  TDataSetSource = class(TDataSource)
  private
    fCds: TClientDataSet;
    fCompsToDoList: TList;
    fTimer: TTimer;
    fDataFolder: string;
    procedure SetActive(const Value: Boolean);
    procedure SetTableName(const Value: string);
    function GetActive: Boolean;
    function GetTableName: string;
  protected
    { Protected declarations }
    procedure TimerProc (Sender: TObject);
  public
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Notification (
      AComponent: TComponent; Operation: TOperation); override;
  published
    property TableName: string read GetTableName write SetTableName;
    property DataFolder: string read fDataFolder write fDataFolder;
    property Active: Boolean read GetActive write SetActive;
  end;

procedure Register;

implementation

uses
  DynDb;

procedure Register;
begin
  RegisterComponents('CodeRageFun', [TDataSetSource]);
end;

{ TDataSetSource }

constructor TDataSetSource.Create(AOwner: TComponent);
begin
  inherited Create (AOwner);
  DataFolder := 'C:\Program Files\Common Files\Borland Shared\Data\';
  fCds := TClientDataSet.Create (self);
  fCds.FileName := DataFolder + 'Customer.cds';
  fCds.Name := 'Internal';
  DataSet := fCds;
  fTimer := TTimer.Create (self);
  fTimer.Enabled := False;
  fTimer.OnTimer := TimerProc;
  fCompsToDoList := TList.Create;
end;

destructor TDataSetSource.Destroy;
begin
  fCompsToDoList.Free;
  fTImer.Free;
  inherited Destroy;
end;

function TDataSetSource.GetActive: Boolean;
begin
  if Assigned (fCds) then
    Result := fCds.Active
  else
    Result := False;
end;

function TDataSetSource.GetTableName: string;
begin
  if Assigned (fCds) then
    Result := ExtractFileName (fCds.FileName)
  else
    Result := '';
end;

procedure TDataSetSource.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;

  if Assigned (AComponent) and
    (AComponent <> self) and
    (Operation = opInsert) and
    IsDataAware (AComponent) then
  begin
    fCompsToDoList.Add (AComponent);
    fTimer.Enabled := True;
  end;
end;

procedure TDataSetSource.SetActive(const Value: Boolean);
begin
  fCds.Active := Value;
end;

procedure TDataSetSource.SetTableName(const Value: string);
begin
  fCds.FileName := DataFolder + Value;
end;

procedure TDataSetSource.TimerProc(Sender: TObject);
var
  Comp: TComponent;
begin
  if fCompsToDoList.Count > 0 then
  begin
    Comp := fCompsToDoList [0];
    // set properties
    if IsDataAware (Comp) then
      ConnectDataSource (Comp, self);
    if HasDataField (Comp) and
        DataSet.Active then
      ConnectDataField (Comp, DataSet.Fields [Random (
          DataSet.FieldCount)].FieldName);
    // remove component from list
    fCompsToDoList.Remove (Comp);
  end;
  if fCompsToDoList.Count = 0 then
    fTimer.Enabled := False;
end;

initialization
  Randomize;
end.
