unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    btnScroll: TButton;
    Image1: TImage;
    tbnBlackHole: TButton;
    procedure btnScrollClick(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure tbnBlackHoleClick(Sender: TObject);
  private
    procedure CaptureScreen;
    procedure HideButtons;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

const
  ScrollSpeed = 120;

procedure TForm1.btnScrollClick(Sender: TObject);
var
  W, H, I, J, LineBytes: Integer;
  Line: PByteArray;
  Bmp: TBitmap;
  R: TRect;
begin
  CaptureScreen;
  HideButtons;

  // get the bitmap of the image
  Bmp := Image1.Picture.Bitmap;
  W := Bmp.Width;
  H := Bmp.Height;

  // allocate enough memory for one line
  LineBytes := Abs (Integer (Bmp.ScanLine [1]) -
    Integer (Bmp.ScanLine [0]));
  Line := AllocMem (LineBytes);

  // scroll how many times as there are lines
  for I := 0 to H - 1 do
  begin
    // copy the first line
    Move ((Bmp.ScanLine [0])^, Line^, LineBytes);
    // for every line
    for J := 1 to H - 1 do
    begin
      // move line to the previous one
      Move ((Bmp.ScanLine [J])^, (Bmp.ScanLine [J-1])^, LineBytes);
      // every nLines update the output
      if (J mod ScrollSpeed = 0) then
      begin
        R := Rect (0, J - ScrollSpeed,
          W, J);
        InvalidateRect (Handle, @R, False);
        UpdateWindow (Handle);
      end;
    end;
    // move back the first line to the end
    Move (Line^, (Bmp.ScanLine [Bmp.Height - 1])^, LineBytes);
    // update the final portion of the bitmap
    R := Rect (0, H - ScrollSpeed,
      W, H);
    InvalidateRect (Handle, @R, False);
    UpdateWindow (Handle);
    // let the program handle other messages
    Application.ProcessMessages;
  end;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  Close;
end;

procedure TForm1.CaptureScreen;
var
  hScreenDc: THandle;
begin
  Image1.Picture.Bitmap := TBitmap.Create;
  Image1.Picture.Bitmap.PixelFormat := pf24bit;
  Image1.Picture.Bitmap.Width := Screen.Width;
  Image1.Picture.Bitmap.Height := Screen.Height;

  hScreenDC := CreateDC ('DISPLAY', nil, nil, nil);
  BitBlt(Image1.Picture.Bitmap.Canvas.Handle,
    0, 0, Screen.Width, Screen.Height,
    hScreenDC, 0, 0, SRCCOPY);

  SetWindowLong (Handle, gwl_Style,
    (GetWindowLong (Handle, gwl_Style) or ws_Popup)
    and not ws_Caption);
  Left := -1;
  Top := -1;
  Width := Screen.Width;
  Height := Screen.Height;
  Image1.Width := Image1.Picture.Bitmap.Width;
  Image1.Height := Image1.Picture.Bitmap.Height;
end;

procedure TForm1.HideButtons;
var
  I: Integer;
begin
  for I := 0 to ControlCount - 1 do
    if Controls [I] is TButton then
      Controls [I].Visible := False;
end;

procedure TForm1.tbnBlackHoleClick(Sender: TObject);
var
  I, X, Y : INteger;
begin
  FormStyle := fsStayOnTop;
  CaptureScreen;
  HideButtons;

  Image1.Canvas.Brush.Color := clBlack;
  for I := 1 to 200 do
  begin
    Application.ProcessMessages;
    x := Random (Screen.Width);
    y := Random(Screen.Height);
    if Image1.Canvas.Pixels [x, y] <> clBlack then
      Image1.Canvas.FloodFill (x, y,
        Image1.Canvas.Pixels [x, y], fsSurface);
  end;
end;

end.
